/*
 * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
 */

/*
 * Copyright (C) 2005-2007 Nokia Corporation.
 * Copyright (C) 2007 INdT.
 *
 * Contact: Mohammad Anwari <Mohammad.Anwari@nokia.com>
 * Contact: Andre Moreira Magalhaes <andrunko@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#ifndef _HILDON_IM_PROTOCOL_H
#define _HILDON_IM_PROTOCOL_H

#include <X11/X.h>

#ifdef __cplusplus
extern "C" {
#endif

#define HILDON_IM_WINDOW_ID_FORMAT                 32
#define HILDON_IM_ACTIVATE_FORMAT                  8
#define HILDON_IM_SURROUNDING_FORMAT               8
#define HILDON_IM_SURROUNDING_CONTENT_FORMAT       8
#define HILDON_IM_INSERT_UTF8_FORMAT               8
#define HILDON_IM_COM_FORMAT                       8
#define HILDON_IM_CLIPBOARD_FORMAT                 32
#define HILDON_IM_CLIPBOARD_SELECTION_REPLY_FORMAT 32

#define HILDON_IM_WINDOW_NAME                    "_HILDON_IM_WINDOW"
#define HILDON_IM_ACTIVATE_NAME                  "_HILDON_IM_ACTIVATE"
#define HILDON_IM_SURROUNDING_NAME               "_HILDON_IM_SURROUNDING"
#define HILDON_IM_SURROUNDING_CONTENT_NAME       "_HILDON_IM_SURROUNDING_CONTENT"
#define HILDON_IM_INSERT_UTF8_NAME               "_HILDON_IM_INSERT_UTF8"
#define HILDON_IM_COM_NAME                       "_HILDON_IM_COM"
#define HILDON_IM_CLIPBOARD_COPY_NAME            "_HILDON_IM_CLIPBOARD_COPY"
#define HILDON_IM_CLIPBOARD_CUT_NAME             "_HILDON_IM_CLIPBOARD_CUT"
#define HILDON_IM_CLIPBOARD_PASTE_NAME           "_HILDON_IM_CLIPBOARD_PASTE"
#define HILDON_IM_CLIPBOARD_SELECTION_QUERY_NAME "_HILDON_IM_CLIPBOARD_SELECTION_QUERY"
#define HILDON_IM_CLIPBOARD_SELECTION_REPLY_NAME "_HILDON_IM_CLIPBOARD_SELECTION_REPLY"

typedef enum
{
  HILDON_IM_MODE,
  HILDON_IM_SHOW,
  HILDON_IM_HIDE,
  HILDON_IM_UPP,
  HILDON_IM_LOW,
  HILDON_IM_TOGGLE,
  HILDON_IM_DESTROY,
  HILDON_IM_CLEAR,
  HILDON_IM_SETCLIENT,
  HILDON_IM_SETNSHOW,
  HILDON_IM_MULTI_LINE,
  HILDON_IM_SINGLE_LINE,
  HILDON_IM_SELECT_ALL,

  /*always last*/
  HILDON_IM_NUM_COMMANDS
} Hildon_IM_Command;

typedef enum
{
  HILDON_IM_CONTEXT_HANDLE_ENTER,
  HILDON_IM_CONTEXT_HANDLE_TAB,
  HILDON_IM_CONTEXT_HANDLE_BACKSPACE,
  HILDON_IM_CONTEXT_HANDLE_SPACE,
  HILDON_IM_CONTEXT_CONFIRM_SENTENCE_START,
  HILDON_IM_CONTEXT_FLUSH_PREEDIT,    /* HWR cleared */
  HILDON_IM_CONTEXT_BUFFERED_MODE,  /* HWR mode */
  HILDON_IM_CONTEXT_DIRECT_MODE,       /* VKB mode */
  HILDON_IM_CONTEXT_INDIRECT_MODE,     /* FKB mode */
  HILDON_IM_CONTEXT_CLIPBOARD_COPY,
  HILDON_IM_CONTEXT_CLIPBOARD_CUT,
  HILDON_IM_CONTEXT_CLIPBOARD_PASTE,
  HILDON_IM_CONTEXT_CLIPBOARD_SELECTION_QUERY,
  HILDON_IM_CONTEXT_AUTOCORRECTION_ON,
  HILDON_IM_CONTEXT_KEYBOARD_ATTACHED,
  HILDON_IM_CONTEXT_KEYBOARD_DETACHED,
  
  /*always last*/
  HILDON_IM_CONTEXT_NUM_COM
} Hildon_IM_Communication;

typedef enum
{
  /* Keyboard mode */
  HILDON_IM_VKB,
  HILDON_IM_FKB,
  HILDON_IM_MODE_MASK           = 0x0fff,

  /* IM context flags. Embedded in here because Hildon_IM_Activate_Message
     can't be grown anymore */
  HILDON_IM_FLAG_BROWSER_WIDGET = 0x1000
} Hildon_IM_Kb_Mode;

typedef enum
{
  HILDON_IM_DIRECT_INSERT,
  HILDON_IM_SET_SURROUNDING
} Hildon_IM_Commit_Mode;

typedef struct
{
  Window input_window;
  Window app_window;
  Hildon_IM_Command cmd;
  int input_mode;
  Hildon_IM_Kb_Mode kb_mode;
} Hildon_IM_Activate_Message;

/*
 * event.xclient.data may not exceed 20 bytes -> Hildon_IM_Insert_Utf8_Message
 * may not exceed 20 bytes. So the maximum size of message buffer is
 * 20 - sizeof(msg_flags) = 16.
 */
#define CLIENT_MESSAGE_BUFFER_SIZE (20 - sizeof(int))

typedef struct
{
  int msg_flag;
  char utf8_str[CLIENT_MESSAGE_BUFFER_SIZE];
} Hildon_IM_Insert_Utf8_Message;

typedef struct
{
  Hildon_IM_Commit_Mode commit_mode;
  int offset_is_relative;
  int cursor_offset;
} Hildon_IM_Surrounding_Message;

typedef struct
{
  int msg_flag;
  char surrounding[CLIENT_MESSAGE_BUFFER_SIZE];
} Hildon_IM_Surrounding_Content_Message;

enum
{
  HILDON_IM_MSG_START,
  HILDON_IM_MSG_CONTINUE,
  HILDON_IM_MSG_END
};

typedef struct
{
  Window input_window;
  Hildon_IM_Communication type;
} Hildon_IM_Com_Message;

typedef enum
{
  HILDON_IM_WINDOW,
  HILDON_IM_ACTIVATE,
  HILDON_IM_INSERT_UTF8,
  HILDON_IM_SURROUNDING,
  HILDON_IM_SURROUNDING_CONTENT,
  HILDON_IM_COM,
  HILDON_IM_CLIPBOARD_COPY, /* for HildonApp */
  HILDON_IM_CLIPBOARD_CUT, /* for HildonApp */
  HILDON_IM_CLIPBOARD_PASTE, /* for HildonApp */
  HILDON_IM_CLIPBOARD_SELECTION_QUERY, /* for HildonApp */
  HILDON_IM_CLIPBOARD_SELECTION_REPLY,

  /*always last*/
  HILDON_IM_NUM_ATOMS
} Hildon_IM_Atom;

#ifdef __cplusplus
}
#endif

#endif
